/*
 * gpp_to_dsp_stub.h
 */

/*
 * Copyright (C) 2010 Texas Instruments Incorporated - http://www.ti.com/ 
 */

/* 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 */
 
#ifndef _GPP_TO_DSP_STUB_H_
#define _GPP_TO_DSP_STUB_H_

// Prevent C++ name mangling
#ifdef __cplusplus
extern far "c" {
#endif


/***********************************************************
* Global Macro Declarations                                *
***********************************************************/

#define RPC_TERMINATE_CACHE_CMDS      (0)
#define RPC_GLOBAL_CACHE_WRITEBACK    (1)
#define RPC_GLOBAL_CACHE_INVALIDATE   (2)
#define RPC_BUFFER_CACHE_WRITEBACK    (3)
#define RPC_BUFFER_CACHE_INVALIDATE   (4)

// Init macro for when the function does not return anything
#define RPC_INIT_VOIDRETURN(F,RT)   \
    uint32_t *__C6RUN_msgPtr__; \
    bool __C6RUN_bGlobalInv__ = false; \
    void *__C6RUN_msg__; \
    C6RUN_libInit(); \
    __C6RUN_msgPtr__ = (uint32_t*) C6RUN_RPC_allocMsgBuff(&__C6RUN_msg__); \
    *__C6RUN_msgPtr__++ = (uint32_t) &FXN_ADDR_ ## F ## _stub;
// Init macros for when the function does return a value
#define RPC_INIT(F,RT) \
    RT __C6RUN_fxnResult__; \
    uint32_t *__C6RUN_msgPtr__,*__C6RUN_msgPtrOrig__; \
    bool __C6RUN_bGlobalInv__ = false; \
    void *__C6RUN_msg__; \
    C6RUN_libInit(); \
    __C6RUN_msgPtrOrig__ = __C6RUN_msgPtr__ = (uint32_t*) C6RUN_RPC_allocMsgBuff(&__C6RUN_msg__); \
    *__C6RUN_msgPtr__++ = (uint32_t) &FXN_ADDR_ ## F ## _stub;

// Macro for performing global cache operations for input buffers             
#define RPC_GLOBAL_CACHE_IN_OPS()   \
    do { \
      *__C6RUN_msgPtr__++=RPC_GLOBAL_CACHE_INVALIDATE; \
      C6RUN_MEM_globalWb(); \
    } while(0)

// Macro for performing cache operations for individual input buffers
#define RPC_BUFFER_CACHE_IN_OPS(P)  \
    do { \
      void *__C6RUN_ptr__; \
      size_t __C6RUN_size__; \
      __C6RUN_ptr__ = C6RUN_MEM_lookupBuffer((void *)P,&__C6RUN_size__); \
      *__C6RUN_msgPtr__++ = (uint32_t) RPC_BUFFER_CACHE_INVALIDATE; \
      *__C6RUN_msgPtr__++ = (uint32_t) C6RUN_MEM_virt2Phys((void *)__C6RUN_ptr__); \
      *__C6RUN_msgPtr__++ = (uint32_t) __C6RUN_size__; \
      C6RUN_MEM_wb(__C6RUN_ptr__,__C6RUN_size__); \
    } while (0)

// Macro for performing global cache operations for output buffers
#define RPC_GLOBAL_CACHE_OUT_OPS() \
    do { \
      *__C6RUN_msgPtr__++=RPC_GLOBAL_CACHE_WRITEBACK; \
      __C6RUN_bGlobalInv__ = true; \
    } while(0)

// Macro for performing cache operations for individual output buffers
#define RPC_BUFFER_CACHE_OUT_OPS(P) \
    do { \
      void *__C6RUN_ptr__; \
      size_t __C6RUN_size__; \
      __C6RUN_ptr__ = C6RUN_MEM_lookupBuffer((void *)P,&__C6RUN_size__); \
      *__C6RUN_msgPtr__++ = (uint32_t) RPC_BUFFER_CACHE_WRITEBACK; \
      *__C6RUN_msgPtr__++ = (uint32_t) C6RUN_MEM_virt2Phys((void *)__C6RUN_ptr__); \
      *__C6RUN_msgPtr__++ = (uint32_t) __C6RUN_size__; \
      C6RUN_MEM_inv(__C6RUN_ptr__,__C6RUN_size__); \
    } while (0)

// Macro to terminate cache operations part of message buffer
#define RPC_TERMINATE_CACHE_OPS()    \
    do { \
      *__C6RUN_msgPtr__++ = RPC_TERMINATE_CACHE_CMDS; \
    } while (0)
    
// Macro for packing in arguments for the remote procedure call
#define RPC_PARAM_PACK(V) \
    do { \
      size_t __C6RUN_size__ = ((sizeof(V) + 3) & (~3u)); \
      memcpy((void *)__C6RUN_msgPtr__,(void *)&V,sizeof(V)); \
      __C6RUN_msgPtr__ = __C6RUN_msgPtr__ + (__C6RUN_size__>>2); \
    } while(0)
    
#define RPC_PARAM_BUFFER_PACK(V) \
    do { \
      *__C6RUN_msgPtr__++ = (uint32_t) C6RUN_MEM_virt2Phys((void *)V); \
    } while(0)

// Macro used for dispatching the remote procedure call
#define RPC_DISPATCH() \
    do { \
      C6RUN_RPC_dispatch(__C6RUN_msg__); \
      if (__C6RUN_bGlobalInv__) \
      { \
        C6RUN_MEM_globalInv(); \
      } \
    } while(0)

// Macros used for returning from stub
#define RPC_RESULT_UNPACK(V) \
    do { \
      int __C6RUN_size__ = ((sizeof(V) + 3) & (~3u)); \
      memcpy((void *)&V,(void *)__C6RUN_msgPtrOrig__,sizeof(V)); \
      __C6RUN_msgPtr__ = __C6RUN_msgPtr__ + (__C6RUN_size__>>2); \
    } while(0)
#define RPC_RETURN() \
    do { \
      C6RUN_RPC_freeMsgBuff(__C6RUN_msg__); \
      return __C6RUN_fxnResult__; \
    } while(0)
#define RPC_RETURN_VOIDRETURN() \
    do { \
      C6RUN_RPC_freeMsgBuff(__C6RUN_msg__); \
      return; \
    } while(0)


/***********************************************************
* Global Struct/Enum/Typedef Declarations                  *
***********************************************************/


/***********************************************************
* Global Variable Declarations                             *
***********************************************************/


/***********************************************************
* Global Function Declarations                             *
***********************************************************/

// C6RunLib init routine
extern int      C6RUN_libInit( void );

// C6Run RPC routines
extern void     C6RUN_RPC_dispatch(void *msg);
extern void*    C6RUN_RPC_allocMsgBuff(void **msg);
extern void     C6RUN_RPC_freeMsgBuff(void *msg);

// C6Run_mem cache routines
extern void     C6RUN_MEM_globalInv( void );
extern void     C6RUN_MEM_globalWb( void );
extern void     C6RUN_MEM_inv(void *ptr,size_t size);
extern void     C6RUN_MEM_wb(void *ptr,size_t size);

// C6Run_mem buffer lookup and translation routines
extern void    *C6RUN_MEM_lookupBuffer(void *ptr, size_t *buffSize);
extern void    *C6RUN_MEM_virt2Phys(void *virtPtr);
extern void    *C6RUN_MEM_phys2Virt(void *physPtr);


/***********************************************************
* End file                                                 *
***********************************************************/

#ifdef __cplusplus
}
#endif
    
#endif /* _GPP_TO_DSP_STUB_H_ */
